from leafhopper.descriptors.descriptor import Descriptor
import json
from urllib.request import urlopen
from leafhopper.logger import logger


def fill_version_if_needed(pkg_dict):
    if "version" not in pkg_dict:
        # folly/catch2/breakpad
        for version_attr in ["version-string", "version-semver", "version-date"]:
            if version_attr in pkg_dict:
                pkg_dict["version"] = pkg_dict[version_attr]


class VcpkgDescriptor(Descriptor):
    def __init__(self) -> None:
        super().__init__()

    def parse(self, vcpkg_json: str) -> list:
        names = self._parse_dependency_names(vcpkg_json)
        pkg_infos = [self._load_pkg_info(name) for name in names]
        return pkg_infos

    def _parse_dependency_names(self, vcpkg_json: str) -> list:
        # parse vcpkg json string into a dictionary
        vcpkg_dict = json.loads(vcpkg_json)
        # get dependencies
        dependencies = vcpkg_dict["dependencies"]
        dep_names = []
        for dep in dependencies:
            # if dep is a string, process it as a dependency name
            # else get `name` attribute from dep dict, and process it as a dependency name
            if isinstance(dep, str):
                dep_name = dep
            else:
                dep_name = dep["name"]
            dep_names.append(dep_name)
        return dep_names

    def _load_pkg_info(self, pkg_name: str) -> dict:
        vcpkg_json_url = f"https://raw.githubusercontent.com/microsoft/vcpkg/master/ports/{pkg_name}/vcpkg.json"
        logger.info(f"processing vcpkg package name={pkg_name}")
        # retrieve the url content via http request
        try:
            pkg_json = urlopen(vcpkg_json_url).read()
            pkg_dict = json.loads(pkg_json)
            fill_version_if_needed(pkg_dict)
        except Exception as e:
            logger.debug(f"failed to load vcpkg package name={pkg_name} vcpkg_json_url={vcpkg_json_url} error={e}")
            pkg_dict = {"name": pkg_name}
        return pkg_dict



from turtle import home
from urllib.request import urlopen
from leafhopper.logger import logger
from leafhopper.descriptors.extra import ExtraInfoLoader, has_github_homepage 
from leafhopper.http.github_http_api import GithubHttpApi

def fill_license(pkg_dict: dict, repo_dict: dict) -> None:
    if not pkg_dict.get("license", None) and "license" in repo_dict:
        license = repo_dict["license"]["name"]
        pkg_dict["license"] = license


def fill_description(pkg_dict: dict, repo_dict: dict) -> None:
    if "description" not in pkg_dict and "description" in repo_dict:
        pkg_dict["description"] = repo_dict["description"]

class GithubInfoLoader(ExtraInfoLoader):
    def __init__(self) -> None:
        self.api = GithubHttpApi() 

    def has_extra_info(self, pkg_dict: dict) -> bool:
        return has_github_homepage(pkg_dict)

    def load(self, pkg_dict: dict, load_license_text: bool) -> dict:
        homepage = pkg_dict["homepage"]
        try:
            repo_dict = self.api.get_repo(homepage)
            fill_license(pkg_dict, repo_dict)
            fill_description(pkg_dict, repo_dict)
            if load_license_text:
                self.fill_license_text(pkg_dict, repo_dict)
            return repo_dict
        except Exception as e:
            logger.debug(f"failed to load info from github name={pkg_dict['name']} homepage={homepage} error={e}")
            return {}

    def fill_license_text(self, pkg_dict: dict, repo_dict: dict) -> None:
        default_branch = repo_dict.get("default_branch", "main")
        full_name = repo_dict["full_name"]
        license_text, license_url = self.api.get_license(full_name, default_branch)
        if license_text:
            pkg_dict["license_text"] = license_text
            pkg_dict["license_url"] = license_url


import re


class ExtraInfoLoader(object):
    def has_extra_info(self, pkg_dict: dict) -> bool:
        return False

    def load(self, pkg_dict: dict) -> dict:
        return pkg_dict


def has_info_missing(pkg_dict, load_license_text=False):
    return (
        has_no_license(pkg_dict)
        or has_no_description(pkg_dict)
        or (load_license_text and has_no_license_text(pkg_dict))
    )


def has_no_license(pkg_dict):
    return "license" not in pkg_dict or not pkg_dict["license"]


def has_no_description(pkg_dict):
    return "description" not in pkg_dict or not pkg_dict["description"]


def has_no_license_text(pkg_dict):
    return "license_text" not in pkg_dict


def has_github_homepage(pkg_dict):
    return "homepage" in pkg_dict and pkg_dict["homepage"].startswith(
        "https://github.com"
    )


def _extract_sourceforge_project_id(homepage):
    patterns = ["sourceforge.net/projects/([^/]+)", "http[s]?://(.*).sourceforge.net.*"]
    for pattern in patterns:
        match = re.search(pattern, homepage)
        if match:
            return match.group(1)
    return None


def extract_sourceforge_project_id(pkg_dict):
    if "homepage" in pkg_dict:
        return _extract_sourceforge_project_id(pkg_dict["homepage"])
    return None



import sys
import os
import argparse
from leafhopper.descriptors.vcpkg import VcpkgDescriptor
from leafhopper.descriptors.poetry import PoetryDescriptor
from leafhopper.descriptors.mvn import MvnDescriptor
from leafhopper.pkg_table_writer import PkgTableWriter
from leafhopper.logger import logger
from leafhopper.extra_pkgs import ExtraPkgs
from leafhopper.combined_license_renderer import CombinedLicenseRenderer
from distutils.util import strtobool
from leafhopper.descriptors.extra.extra_info_loader import load_extra_info_for_pkgs
from leafhopper.license_resolver import LicenseResolver

# import importlib.metadata
# package_metadada = importlib.metadata.metadata("leafhopper")
# # info from pyproject.toml's `version` and `description`
# LEAFHOPPER_VERSION = package_metadada.get("Version")
# LEAFHOOPER_SUMMARY = package_metadada.get("Summary")


def _leafhopper_parser():
    parser = argparse.ArgumentParser(prog="leafhopper")

    parser.add_argument(
        "file",
        nargs="+",
        help="specify the path to the project descriptor, vcpkg.json are supported",
    )

    parser.add_argument(
        "-f",
        "--format",
        default="markdown",
        help=f"specify the output format, {PkgTableWriter.supported_formats()} are supported",
    )

    parser.add_argument(
        "-o",
        "--output",
        help=f"specify a file for the output, default is stdout",
    )

    parser.add_argument(
        "-l",
        "--logging-level",
        help=f"specify the logging level, debug|info|warning|error|critical are supported. Default is `info`",
    )

    parser.add_argument(
        "-c",
        "--columns",
        help=f"""specify the output table header. A comma separated column names, 
built-in column names are 'name,version,homepage,license,description'. 
You can change the header to alter the column orders, 
and non built-in names specifed in the column will be shown as empty.
""",
    )

    parser.add_argument(
        "-e",
        "--extra",
        help=f"optional. specify a json file for extra pkgs to override the pkgs in the descriptor",
    )

    parser.add_argument(
        "-b",
        "--combine",
        action='store_true',
        help=f"optional. whether to generate a combined license. Specify the argument to toggle it. Default is false",
    )
    return parser


def parse_sys_args(sys_args):
    parser = _leafhopper_parser()
    args = parser.parse_args(sys_args)
    return vars(args)


def _validate_format(format):
    supported_formats = PkgTableWriter.supported_formats()
    if format not in supported_formats:
        raise ValueError(
            f"Given format {format} is not supported. supported_formats={supported_formats}"
        )


def _read_descriptor(file):
    with open(file, "r") as f:
        content = f.read()
        return content


def _get_descriptor(file):
    base_name = os.path.basename(file)
    descriptors = {
        "vcpkg.json": VcpkgDescriptor,
        "pyproject.toml": PoetryDescriptor,
        "pom.xml": MvnDescriptor,
    }
    if base_name in descriptors:
        return descriptors[base_name]
    else:
        raise ValueError(
            f"Unsupported descriptor {file}. Only vcpkg.json and pyproject.toml files are supported"
        )


def _write_pkg_table(pkg_infos, table_writer, output, columns):
    if output:
        if isinstance(output, str):
            with open(output, "w") as f:
                table_writer.write(pkg_infos, f, columns)
        else:  # StringIO
            table_writer.write(pkg_infos, output, columns)
    else:
        table_writer.write(pkg_infos, columns=columns)


def process_descriptors(
    files, format, output, columns=None, extra_pkgs_file=None, combined_license=False
):
    _validate_format(format)
    table_writer = PkgTableWriter(format)
    all_pkg_infos = []
    license_resolver = LicenseResolver()
    for file in files:
        descriptor = _get_descriptor(file)()
        # open file and read content
        descriptor_content = _read_descriptor(file)
        pkg_infos = descriptor.parse(descriptor_content)
        # override pkg_infos with provided overrides
        if extra_pkgs_file:
            logger.info(f"loading extra package information extra_file={extra_pkgs_file} current_pkgs={len(pkg_infos)}")
            extra_pkgs = ExtraPkgs(extra_pkgs_file)
            pkg_infos = extra_pkgs.override_pkg_infos(pkg_infos)
            logger.info(f"overrideed package information pkg_infos={len(pkg_infos)}")
        license_resolver.resolve(pkg_infos)
        load_extra_info_for_pkgs(pkg_infos, combined_license)
        all_pkg_infos.extend(pkg_infos)
        _write_pkg_table(pkg_infos, table_writer, output, columns)
    if combined_license:
        combined_license_renderer = CombinedLicenseRenderer()
        combined_licenses_txt = combined_license_renderer.render(all_pkg_infos)
        # write combined licenses to a file called `LICENSES.txt`
        open("LICENSES.txt", "w").write(combined_licenses_txt)


def _map_logging_level(level):
    levels = {
        "debug": 10,
        "info": 20,
        "warning": 30,
        "error": 40,
        "critical": 50,
    }
    return levels.get(level, 20)


def _set_logging_level(level):
    logger.setLevel(_map_logging_level(level))


def _get_columns(columns: list) -> list:
    # split columns by comma and trim each value
    if columns and columns.strip():
        columns = columns.split(",")
        columns = [col.strip() for col in columns]
    else:
        columns = None
    return columns


def main():
    args = parse_sys_args(sys.argv[1:])
    files = args["file"]
    format = args["format"]
    extra_pkgs_file = args["extra"]
    combined_license = args["combine"]
    columns = _get_columns(args["columns"])

    logging_level = args["logging_level"]
    _set_logging_level(logging_level)
    output = args.get("output", None)
    process_descriptors(
        files, format, output, columns, extra_pkgs_file, combined_license
    )


if __name__ == "__main__":
    main()



from leafhopper.descriptors.descriptor import Descriptor
import json
from urllib.request import urlopen
from leafhopper.logger import logger
import tomli


def _get_home_page(pypi_info: dict) -> None:
    if "home_page" in pypi_info and pypi_info["home_page"]:
        return pypi_info["home_page"]
    elif "project_urls" in pypi_info and "Homepage" in pypi_info["project_urls"]:
        return pypi_info["project_urls"]["Homepage"]
    elif "project_urls" in pypi_info and "Source Code" in pypi_info["project_urls"]:
        return pypi_info["project_urls"]["Source Code"]
    else:
        return None


def _load_pkg_info(pkg_info: dict) -> dict:
    pkg_name = pkg_info["name"]
    pypi_project_url = f"https://pypi.org/pypi/{pkg_name}/json"
    logger.info(f"processing pypi package name={pkg_name} url={pypi_project_url}")
    # retrieve the url content via http request
    try:
        pkg_json = urlopen(pypi_project_url).read()
        pkg_dict = json.loads(pkg_json)
        pypi_info = pkg_dict["info"]
        pkg_info["homepage"] = _get_home_page(pypi_info)
        pkg_info["description"] = pypi_info.get(
            "summary", pypi_info.get("description", None)
        )
        pkg_info["license"] = pypi_info.get("license", None)
    except Exception as e:
        logger.debug(f"failed to load pypi package name={pkg_name} error={e}")
    return pkg_info


class PoetryDescriptor(Descriptor):
    def __init__(self) -> None:
        super().__init__()

    def parse(self, poetry_toml: str) -> list:
        pkg_infos = self._parse_dependencies(poetry_toml)
        pkg_infos = [_load_pkg_info(pkg_info) for pkg_info in pkg_infos]
        return pkg_infos

    def _parse_dependencies(self, poetry_toml: str) -> list:
        poetry_dict = tomli.loads(poetry_toml)
        pkg_infos = []
        if (
            "tool" in poetry_dict
            and "poetry" in poetry_dict["tool"]
            and "dependencies" in poetry_dict["tool"]["poetry"]
        ):
            dependencies = poetry_dict["tool"]["poetry"]["dependencies"]
            for dep_name, dep_info in dependencies.items():
                if dep_name != "python":
                    pkg_info = {"name": dep_name}
                    if isinstance(dep_info, str):
                        pkg_info["version"] = dep_info
                    elif isinstance(dep_info, dict):
                        if "version" in dep_info:
                            pkg_info["version"] = dep_info["version"]
                    pkg_infos.append(pkg_info)
        return pkg_infos



import os
import urllib3
from leafhopper.logger import logger
import json


class GithubHttpApi(object):
    # get username and password from environment variables
    def __init__(self):
        self.username = os.environ.get("LEAFHOPPER_GITHUB_USERNAME")
        self.password = os.environ.get("LEAFHOPPER_GITHUB_PASSWORD")
        self.http = urllib3.PoolManager()

    def _get(self, url):
        headers = {}
        if self.username:
            headers = urllib3.util.make_headers(
                basic_auth=f"{self.username}:{self.password}"
            )
        res = self.http.request("GET", url, headers=headers)
        if res.status == 200:
            return res.data.decode("utf-8")
        else:
            raise Exception(f"failed to call github api status={res.status} response={res.data}")

    def get_repo(self, homepage: str):
        # https://docs.github.com/en/rest/guides/getting-started-with-the-rest-api#repositories
        github_repo_api_url = homepage.replace("github.com", "api.github.com/repos")
        logger.info(f"loading repo info from github url={github_repo_api_url}")
        repo_json = self._get(github_repo_api_url)
        repo_dict = json.loads(repo_json)
        return repo_dict

    def get_license(self, repo_full_name: str, default_branch: str):
        for license_file in ["LICENSE", "LICENSE.txt", "LICENSE.md", "LICENSE.rst"]:
            license_url = f"https://raw.githubusercontent.com/{repo_full_name}/{default_branch}/{license_file}"
            logger.debug(
                f"loading github license text name={repo_full_name} url={license_url}"
            )
            try:
                license_text = self._get(license_url)
                return (license_text, license_url)
            except Exception as e:
                logger.debug(
                    f"failed to load github license text repo={repo_full_name} url={license_url} error={e}"
                )
        return (None, None)



from jinja2 import Environment, PackageLoader, select_autoescape

class CombinedLicenseRenderer(object):
    def render(self, pkg_infos: list):
        env = Environment(
            loader=PackageLoader("leafhopper", "templates"),
            autoescape=select_autoescape()
        )
        template = env.get_template("LICENSES.txt.template")
        license_txt = template.render(pkg_infos=pkg_infos)
        return license_txt


from urllib.request import urlopen
from leafhopper.logger import logger

class LicenseResolver(object):
    def resolve(self, pkg_infos: list):
        for pkg_info in pkg_infos:
            if "license_text" in pkg_info:
                return
            if "license_url" in pkg_info:
                license_url = pkg_info["license_url"]
                logger.info(f"loading license text from url name={pkg_info['name']} url={license_url}")
                try:
                    license_text = urlopen(license_url).read().decode("utf-8")
                    pkg_info["license_text"] = license_text
                    return
                except Exception as e:
                    logger.error(f"failed to load license text from url name={pkg_info['name']} url={license_url} error={e}")



from leafhopper.descriptors.descriptor import Descriptor

from io import BytesIO, StringIO
try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    from xml.etree import ElementTree
import json
from urllib.request import urlopen
from leafhopper.logger import logger

NS = "http://maven.apache.org/POM/4.0.0"

def _parse_ignoring_namespace(pom):
    if isinstance(pom, str):
        xml = StringIO(pom)
    else:
        xml = BytesIO(pom)
    it = ElementTree.iterparse(xml)
    for _, el in it:
        _, _, el.tag = el.tag.rpartition('}') # strip ns
    return it.root

def _get_pom_path(pkg_info: dict) -> str:
    group_id = pkg_info["group_id"].replace(".", "/")
    artifact_id = pkg_info["artifact_id"]
    version = pkg_info["version"]
    return "/".join([group_id, artifact_id, version, f"{artifact_id}-{version}.pom"])


def _fill_homepage(pkg_info: dict, tree) -> dict:
    homepage = tree.find(f"url")
    if homepage is not None:
        pkg_info["homepage"] = homepage.text
    else:
        scm_url = tree.find(f"scm/url")
        if scm_url is not None:
            pkg_info["homepage"] = scm_url.text

def _load_pkg_info_with_version(pkg_info: dict) -> dict:
    pom_path = _get_pom_path(pkg_info)
    pom_url = f"https://search.maven.org/remotecontent?filepath={pom_path}"
    # get pom via http request
    try:
        logger.debug(f"loading pom url={pom_url}")
        pom = urlopen(pom_url).read()
        tree = _parse_ignoring_namespace(pom)
        _fill_homepage(pkg_info, tree)
        
        description = tree.find(f"description")
        if description is not None:
            pkg_info["description"] = description.text
        licenses = tree.findall(f"licenses/license/name")
        if licenses:
            license_texts = [license.text for license in licenses]
            pkg_info["license"] = ", ".join(license_texts)
    except Exception as e:
        logger.debug(f"failed to load pom url={pom_url} error={e}")
    return pkg_info


def _load_pkg_version(pkg_info: dict) -> dict:
    artifact_url = f"https://search.maven.org/solrsearch/select?q=g:{pkg_info['group_id']}+AND+a:{pkg_info['artifact_id']}&core=gav&rows=10&wt=json"
    logger.debug(f"loading artifact version url={artifact_url}")
    artifact_versions_content = urlopen(artifact_url).read()
    artifact_versions = json.loads(artifact_versions_content)
    if "response" in artifact_versions and "docs" in artifact_versions["response"]:
        docs = artifact_versions["response"]["docs"]
        if len(docs) > 0:
            pkg_info["version"] = docs[0]["v"]
    return pkg_info


def _load_pkg_info(pkg_info: dict) -> dict:
    pkg_name = pkg_info["name"]
    logger.info(f"processing maven package name={pkg_name}")
    if not pkg_info["version"]:
        _load_pkg_version(pkg_info)
        if not pkg_info["version"]:
            return pkg_info
    return _load_pkg_info_with_version(pkg_info)


class MvnDescriptor(Descriptor):
    def __init__(self) -> None:
        super().__init__()

    # use element tree API to parse `dependencies` from maven pom file
    def parse(self, pom: str) -> list:
        # parse pom xml string into element tree
        tree = _parse_ignoring_namespace(pom)
        # find the `dependencies` element
        dependencies = tree.find(f"dependencies")
        deps = []
        # iterate through each `dependency` element
        for dependency in dependencies:
            pkg_info = {}
            # get groupId, artifactId, version
            group_id = dependency.find(f"groupId").text
            artifact_id = dependency.find(f"artifactId").text
            version_element = dependency.find(f"version")
            if version_element:
                version = version_element.text
            else:
                version = None
            # create a key-value pair for the dictionary
            pkg_info["name"] = group_id + "." + artifact_id
            pkg_info["group_id"] = group_id
            pkg_info["artifact_id"] = artifact_id
            pkg_info["version"] = version

            _load_pkg_info(pkg_info)
            deps.append(pkg_info)
        return deps



from pytablewriter import (
    MarkdownTableWriter,
    HtmlTableWriter,
    CsvTableWriter,
    JsonTableWriter,
    LatexTableWriter,
)

WRITER_MAP = {
    "md": MarkdownTableWriter,
    "markdown": MarkdownTableWriter,
    "html": HtmlTableWriter,
    "csv": CsvTableWriter,
    "json": JsonTableWriter,
    "latex": LatexTableWriter,
}


class PkgTableWriter(object):
    def __init__(self, format: str):
        self.format = format
        if format not in WRITER_MAP:
            raise ValueError(f"{format} is not supported")

    def write(self, pkg_infos: list, output=None, columns=None):
        table_name = "Package Dependencies"
        pkg_attrs = ["name", "version", "homepage", "license", "description"]
        if columns:
            pkg_attrs = columns
        # retrieve package attributes from pkg_infos
        pkg_list = [self._get_attrs(pkg_info, pkg_attrs) for pkg_info in pkg_infos]
        writer = WRITER_MAP[self.format]()
        writer.table_name = table_name
        writer.headers = pkg_attrs
        writer.value_matrix = pkg_list
        if output:
            writer.stream = output

        writer.write_table()
        writer.write_null_line()

    def _get_attrs(self, pkg_info: dict, attrs: list):
        return [pkg_info.get(attr, "") for attr in attrs]

    @staticmethod
    def supported_formats():
        return list(WRITER_MAP.keys())



from urllib.request import urlopen
import json
from leafhopper.logger import logger
from leafhopper.descriptors.extra import ExtraInfoLoader
from leafhopper.descriptors.extra import has_no_license, extract_sourceforge_project_id


def fill_license(pkg_dict: dict, project_dict: dict) -> None:
    if not pkg_dict.get("license", None):
        if "categories" in project_dict and "license" in project_dict["categories"]:
            license = project_dict["categories"]["license"]
            if isinstance(license, list):
                pkg_dict["license"] = license[0]["shortname"]

def fill_description(pkg_dict: dict, project_dict: dict) -> None:
    if "description" not in pkg_dict:
        pkg_dict["description"] = project_dict.get(
            "short_description", project_dict.get("summary", None)
        )


class SourceForgeInfoLoader(ExtraInfoLoader):
    def __init__(self, project_id = None) -> None:
        self.project_id = project_id

    def has_extra_info(self, pkg_dict: dict) -> bool:
        self.project_id = extract_sourceforge_project_id(pkg_dict)
        return self.project_id

    def load(self, pkg_dict: dict, load_license_text: bool = False) -> dict:
        assert self.project_id is not None
        # https://sourceforge.net/p/forge/documentation/Allura%20API/
        project_api_url = f"https://sourceforge.net/rest/p/{self.project_id}"
        try:
            logger.info(
                f"loading info from sourceforge name={pkg_dict['name']} url={project_api_url}"
            )
            sourceforge_json = urlopen(project_api_url).read()
            project_dict = json.loads(sourceforge_json)
            fill_license(pkg_dict, project_dict)
            fill_description(pkg_dict, project_dict)
        except Exception as e:
            logger.debug(f"failed to load info from sourceforge name={pkg_dict['name']} url={project_api_url} error={e}")
        return pkg_dict



from leafhopper.descriptors.extra.github_info_loader import GithubInfoLoader
from leafhopper.descriptors.extra.sourceforge_info_loader import SourceForgeInfoLoader
from leafhopper.descriptors.extra import has_info_missing


def load_extra_info(pkg_dict: dict, load_license_text=False) -> dict:
    if has_info_missing(pkg_dict, load_license_text):
        loaders = [GithubInfoLoader(), SourceForgeInfoLoader()]
        for loader in loaders:
            if loader.has_extra_info(pkg_dict):
                return loader.load(pkg_dict, load_license_text)
    return pkg_dict

def load_extra_info_for_pkgs(pkg_list: list, load_license_text=False) -> list:
    return [load_extra_info(pkg, load_license_text) for pkg in pkg_list]


import json


class ExtraPkgs(object):
    def __init__(self, extra_pkgs_file):
        self.extra_pkgs_file = extra_pkgs_file
        with open(self.extra_pkgs_file, "r") as f:
            extra_info = json.load(f)
            self.overrides = self._read_overrides(extra_info)
            self.hiddens = set(extra_info.get("hiddens", []))

    def _read_overrides(self, extra_info) -> dict:
        if "overrides" in extra_info:
            overrides = extra_info["overrides"]
            return {pkg["name"]: pkg for pkg in overrides}
        else:
            return {}

    def override_pkg_infos(self, pkg_infos: list) -> list:
        for pkg_info in pkg_infos:
            if pkg_info["name"] in self.overrides:
                # mark pkg_info as overridden
                self.overrides[pkg_info["name"]]["_overridden"] = True
                override_pkg = self.overrides[pkg_info["name"]]
                # get all properties from override_pkg
                attrs = set(override_pkg.keys()).union(
                    {"version", "license", "homepage", "description"}
                )
                for attr in attrs:
                    if attr in override_pkg:
                        pkg_info[attr] = override_pkg[attr]
                if "overridden_name" in override_pkg:
                    pkg_info["name"] = override_pkg["overridden_name"]

        # hide packages
        pkg_infos = [p for p in pkg_infos if p["name"] not in self.hiddens]
        return pkg_infos



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leafhopper',
 'leafhopper.descriptors',
 'leafhopper.descriptors.extra',
 'leafhopper.http']

package_data = \
{'': ['*'], 'leafhopper': ['templates/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'pytablewriter[html]>=0.64.2,<0.65.0',
 'tomli>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['leafhopper = leafhopper.main:main']}

setup_kwargs = {
    'name': 'leafhopper',
    'version': '0.5.1',
    'description': 'A command line tool for generating project dependencies table',
    'long_description': "# leafhopper\nDo you get asked by your employer to provide a list of open source libraries that you use in the project for legal review?\n\n`leafhopper` is a command line tool used for generating a table of dependencies for a project, including their licenses, so that you don't have to manually maintain such a list for every release of your project.\n\n# How it works\nThe tool parses the project descriptor, based on different project types (`poetry`/`maven`/`vcpkg` are supported currently), and generates a table of dependencies. When some critical information, such as license, is not available in the project descriptor, `leafhopper` will test if this is a github/sourceforge project and try loading relevant information from `github.com`/`sourceforge.net`.\n\n# Features\n* parse multiple different project types to generate a table of dependencies from them\n* load license information from github/sourceforge\n* support overriding the list of dependencies from the project descriptor when you cannot get correct information from the project descriptor\n* support customizing the output columns\n* multiple outout formats\n* generate a combined license file from all the licenses of the dependencies\n* github token can be provided in environment variable to avoid github API rate limiting\n\n# Installation\n```\npip install leafhopper\n```\n\n# Usage\n```\nleafhopper /path/to/project/descriptor\n```\n\n## arguments\n* `--format`: the format of the output. Possible values are `markdown`/`html`/`json`/`latex`/`csv`. Default is `markdown`.\n* `--output`: the output file path. If not specified, the output will be printed to stdout.\n* `--columns`: the output table header columns. It is a comma separated string. Default value is `name,version,homepage,license,description`. You can change the order of columns or add empty columns by changing the value. For example, `name,license,homepage,component` add a new empty column called `component` and reorder the columns as well.\n* `--logging-level`: the logging level. Possible values are `debug`/`info`/`warning`/`error`/`critical`. Default is `info`. \n  * Set the logging level to above `info` (e.g. `error`) to supress non critical messages so that only table is printed to stdout (if no output file is specified).\n  * Set the logging level to `debug` to enable debug messages.\n* `--extra`: the file path to a JSON file path containing extra package information to override the information parsed from project descriptors. The `overrides` property in JSON file is an array of objects with the following properties (here is an [example](tests/data/extra.json)):\n  * `name`\n  * `version`, optional\n  * `license`, optional\n  * `homepage`, optional\n  * `description`, optional\n  * `disclosed_source`, optional, a url to the discolosed source for some license requirement such as MPL\n* `--combine`: whether to generate a combined license file. Use `true`/`false` to toggle it. Default is false.\n* `--help`: show the help message\n\n## examples\n1. extract `pyproject.toml` dependencies with markdown format and save it into `dependencies.md` file\n```\nleafhopper /path/to/pyproject.toml --output=dependencies.md\n```\n\n2. extract `pom.xml` dependencies with html format\n```\nleaphopper /path/to/pom.xml --format=html\n```\n\n3. suppress logging and output to stdout and use CLI tool [`glow`](https://github.com/charmbracelet/glow) to display it\n```\nleafhopper /path/to/vcpkg.json --format md --logging-level error | glow -\n```\n\n4. use custom columns to change the column order and add an empty column called `component`, which you can fill later on\n```\nleaphopper /path/to/pom.xml --columns name,component,version,license,homepage,description\n```\n\n5. use an extra JSON file to override the information parsed from project descriptors\n```\nleaphopper /path/to/pom.xml --extra=tests/data/extra.json\n```\n\n6. generate a combined license file so that you can put it as part of your project\n```\nleaphopper /path/to/pom.xml --combine=true\n```\nIt will generate a file called `LICENSES.txt` with all the licenses information of the dependencies.\n\n7. [advanced] when providing both `--extra` and `--columns`, you can get any information from the extra JSON file to be shown in the output table. For example, if you would like to add a `disclosed_source` column, you can specify this property in the `extra.json` file and specify the `disclosed_source` column in the `--columns` argument.\n```\nleaphopper /path/to/vcpkg.json --extra=/extra/with/disclosed/sources/extra.json --columns name,version,homepage,license,description,disclosed_source\n```\n\n# Supported formats\n* markdown\n* LaTex\n* html\n* json\n* csv\n## sample output\n* markdown format output\n```markdown\n# Package Dependencies\n|      name       |version|           homepage            | license  |                               description                               |\n|-----------------|-------|-------------------------------|----------|-------------------------------------------------------------------------|\n|simdjson         |2.2.0  |https://simdjson.org/          |Apache-2.0|A extremely fast JSON library that can parse gigabytes of JSON per second|\n|pcre             |   8.45|https://www.pcre.org/          |          |Perl Compatible Regular Expressions                                      |\n|pugixml          |1.12.1 |https://github.com/zeux/pugixml|MIT       |Light-weight, simple and fast XML parser for C++ with XPath support      |\n|arrow            |8.0.0  |https://arrow.apache.org       |Apache-2.0|Cross-language development platform for in-memory analytics              |\n```\n\n# Supported project types\n* poetry project described by `pyproject.toml`\n    * https://python-poetry.org/docs/pyproject/    \n* maven project described by `pom.xml`\n    * https://maven.apache.org/pom.html\n    * `pom.xml` with or without Maven XML namespace are supported.\n* vcpkg project described by `vcpkg.json`\n    * https://vcpkg.readthedocs.io/en/latest/specifications/manifests/\n* more project types such as npm will be supported in the future\n\n# Github API rating limit\nIf you have a really large project, you may encounter github API rate limiting. You can provide a github personal token in the environment variables to avoid this:\n  * `LEAFHOPPER_GITHUB_USERNAME`\n  * `LEAFHOPPER_GITHUB_PASSWORD`\n  You can export these variables to your shell environment like this:\n  ```\n  export LEAFHOPPER_GITHUB_USERNAME={your_github_user_name}\n  export LEAFHOPPER_GITHUB_PASSWORD={your_personal_token} # something like `ghp_pciFGDQlXAqDGNBXHsUbHHHZly7cf71ZKEVg`\n  ```\nTo create a Github personal token, follow the instructions [here](https://docs.github.com/en/authentication/keeping-your-account-and-data-secure/creating-a-personal-access-token)\n  * Unless you need to access some private project information in Github, you can keep the personal token's permission minimal in Github since it only needs to be authencated against Github so that you can use higher the rate limit (see more details [here](https://docs.github.com/en/rest/overview/resources-in-the-rest-api#requests-from-personal-accounts)).\n\n# Changelog\n[Changelog](CHANGELOG.md)\n\n# Known issues\n* Some open source libraries, doesn't have the license information available in the project descriptor (or in `github.com`/`sourceforge.net`), and the cell will be blank and you have to manually fill it.\n\n# TODO\n* Support more project types, such as `npm`'s `package.json` and `pip`'s `requirements.txt`\n",
    'author': 'Yue Ni',
    'author_email': 'niyue.com@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/niyue/leafhopper',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
